/*
 * ip4range2cidr
 * 
 * SYNOPSYS
 * Converts an IPv4 continuous range to a set of CIDR notation.
 * 
 * DESCRIPTION
 * Given a range of IPv4 address (init-end), the program
 * converts this rage into one or more CIDR notation. It
 * dont care about class of IP address and identifies
 * the smallest set of CIDR for that IP range. So, the
 * program is sensible to initial IP address.
 * The output data shows the network ID and the netmask:
 * number of bits and IP address formats.
 *
 * AUTHOR
 * Ulisses Guedes - August/2006 
 *
 * LICENSE
 * 	GPL
 *
 *
 * VERSION
 * 	1.0 - August/2006
 *
 * REALEASE NOTES
 * - There is no arguments. Usage is simple:
 * ip4range2cidr IP1 IP2
 *
 * - Include the comments: NETWORK, NETMASK and BROADCAST
 * - Validate input data
 *   
 */
#include <stdio.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <unistd.h>
int cidr,html;

void HtmlHdr(int i, char *ip1, char *ip2)
{
	if (i == 1) printf("Content-Type: text/html\n\n");
	printf("<html><body>\n");
	printf("<br>IP Range:  %s - %s\n",ip1,ip2);
	printf("<br><br>CIDR Answer:<br>\n");
}

void VePrin(unsigned int ip, unsigned int msk )
{
	unsigned int mk,i;
	union IPK {
		unsigned char b[4];
		unsigned int  i;
	} ipk;
	
	ipk.i=ip;
	if (cidr) 
		printf("NET/MASK=%d.%d.%d.%d/",ipk.b[3],ipk.b[2],ipk.b[1],ipk.b[0]);
	else
		printf("NETWORK=%d.%d.%d.%d / ",ipk.b[3],ipk.b[2],ipk.b[1],ipk.b[0]);
		
	i=0; mk=msk;
	while (	(mk !=0) && (i<32)) {
		i++;
		mk=mk<<1;
	}
	ipk.i=msk;
	if (cidr) {
		printf("%d or %d.%d.%d.%d \n",i,ipk.b[3],ipk.b[2],ipk.b[1],ipk.b[0]);
		return;
	}
	else
		printf("NETMASK=%d or %d.%d.%d.%d / ",i,ipk.b[3],ipk.b[2],ipk.b[1],ipk.b[0]);
	mk=(ip & msk) + ~msk;
	ipk.i=mk;
	
	if (!cidr)  printf("BROADCAST=%d.%d.%d.%d\n",ipk.b[3],ipk.b[2],ipk.b[1],ipk.b[0]);	
}

void Usage(char *c)
{
	printf("Usage: %s IPi IPf [cidr|html|nhtml]\n",c);
	printf("where\n");
	printf("IPi - first IP address of range (MUST BE THE SMALLEST VALUE!)\n");
        printf("IPf - last IP address of range (Must be a valid broacast for any subnet)\n");
	printf("cidr - if cidr is present then the output will be printed as\n");
	printf("       NETWORK/NETMASK together otherwise the output will print\n");
	printf("       isolated but at the same line\n");
	printf("html - the program will output a html header\n");
	printf("       and the results at cidr format.\n");
	printf("nhtml - the program will output only the results at cidr\n");
	printf("        encapsulated under html.\n");
	printf("no complement - prints Network, mask and broadcast addresses.\n");
	printf("NOTE:  The IP must be represented in decimal dot number\n\n");
	printf("\nThe worst thing in life is to know that she is running out ");
	printf("quickly and the time left is insufficient to transfer the little ");
	printf("knowledge that I have learned. Life ends but knowledge can stay ");
	printf("forever and continue by others. 'The founder of GURI'\n\n");
	printf("Note: Do not worry. He was in the toilet for two days with diarrhea.\n");
	printf("He survived! Augh!! He's still alive!!\n\n\n");
	printf("    ip4range2cidr - ipv4 address range to CIDR converter\n");
	printf("    Copyright (C) 2013 GURI SoftHouse LTDA\n\n");

	printf("    This program is free software: you can redistribute it and/or modify\n");
	printf("    it under the terms of the GNU General Public License as published by\n");
	printf("    the Free Software Foundation, either version 3 of the License, or\n");
	printf("    (at your option) any later version.\n\n");

	printf("    This program is distributed in the hope that it will be useful,\n");
	printf("    but WITHOUT ANY WARRANTY; without even the implied warranty of\n");
	printf("    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n");
	printf("    GNU General Public License for more details.\n\n");

	printf("    You should have received a copy of the GNU General Public License\n");
	printf("    along with this program.  If not, see <http://www.gnu.org/licenses/>.\n\n");

	printf("    techinfo@guri.no-ip.org.\n\n");

}

int main(int argc, char *argv[])
{
	unsigned int ip,ip1,ip2,ip1old;
	unsigned int msk=0xFFFFFFFF,mk;
	unsigned int msk0=0,msk1;
	unsigned int brd;
	int i=31,j=0,k;
	char valid[]=".0123456789";
	
	if (argc < 3) {
		Usage(argv[0]);
		return 1;
	}
	/* IP inicial */
	ip = inet_addr(argv[1]);
	ip1 = ntohl(ip);
	/* IP final */
	ip = inet_addr(argv[2]);
	ip2=ntohl(ip);
	
	if (ip1 > ip2) {
		printf("Invalid first ip-address. It is greater than the second ip\n");
		Usage(argv[0]);	
		return 1;
	}
	
	if (argc > 3) {
		cidr=0;
		html=0;
		if (strncmp(argv[3],"cidr",4) == 0) {
			
			cidr=1;
			
		} else if (strncmp(argv[3],"html",4) == 0) {
			
			cidr=1;
			html=1;
			
		} else if (strncmp(argv[3],"nhtm",4) == 0) {
			
			cidr=1;
			html=2;
		}
	}
	
	if (html) HtmlHdr(html,argv[1],argv[2]);
	
	do {
		/* Compute what mask is acceptable for ip1 */
		while ((( ip1 & msk0) != ip1) && (j<33)) {
			msk0 = (msk0 >>1)|0x80000000;
			j++;
		}
		/* Seek for the smallest mask */
		msk1 = msk0;
		k=j;
		while ((( ip1 & msk1) == ip1) && (k>0)) {
			msk1 = (msk1 <<1);
			k--;
		}
		brd = (ip1 & msk1) + ~msk1;
		if (brd < ip2) {
			msk0 = (msk1 >> 1) | 0x80000000;
			msk1=msk0;
			k++;
			j=k+1;
			brd = (ip1 & msk0) + ~msk0;
			VePrin(ip1, msk0 );
			if (html) printf("<br>\n");
		}
	       	else {
			msk0=(msk1 >> 2) | 0xC0000000;
			j=k+2;
		}
		ip1old=ip1;
		ip1 = brd+1;
	} while ((brd < ip2) && (ip1 < ip2));
	
	/* sometimes ip1 must be adjusted with old value */
	if (ip1 >= ip2) ip1=ip1old;
	
	/* Compute the minimum netmask */
	while (((ip2 & msk) > ip1) && (i>0)) {
	       	msk = msk << 1;
		i--;
	}
	
	if (msk0 > msk) {
		do {
			brd = (ip1 & msk0) + ~msk0;
			if (brd == ip1) ip1 = brd+1;
			else break;
		} while (1);
	}
	
	/*  Compute:
	 *  - the IP broadcst
	 *  - new mask of the new segment
	 *  - new network ID  */
	do {
		brd = (ip1 & msk)+ (~msk);
		if (brd > ip2) {
		       	msk = msk >> 1;
			msk = msk | 0x80000000;
			i++;
		}
		if (brd < ip2) {
			/* Print parcial results */
			VePrin(ip1, msk );
			if (html) printf("<br>\n");
			ip1=brd+1;
			msk = (msk >> 1) | 0x80000000;
		}
	} while (brd != ip2);
	
	/* Print Final block */
	VePrin(ip1, msk );
	
	if (html) printf("<br></body></html>\n");
	return 0;
}

